const path = require('path');
const postModels = require('../models/post');

function checkPostExistence(id, bool = true) {
    var query = "SELECT * FROM posts WHERE id = ?";
    return new Promise((resolve, reject) => {
        pool.query(query, [id], (err, results) => {
            if (err) {
                resolve(false);
            } else {
                if (bool == false) {
                    resolve(results[0]);
                } else {
                    resolve(true);
                }
            }
        });
    });
}

function checkLikeExistence(post_id, user_id) {
    var query = "SELECT * FROM posts_likes WHERE user_id = ? and post_id = ?";
    return new Promise((resolve, reject) => {
        pool.query(query, [user_id, post_id], (err, results) => {
            if (err) {
                resolve(false);
            } else if (!results[0]) {
                resolve(false);
            } else {
                resolve(true);
            }
        });
    });
}

function uploadMedia(post_id, file) {
    var ext = path.extname(file.originalname);
    if (ext !== '.png' && ext !== '.jpg' && ext !== '.pdf' && ext !== '.jpeg') {
        return false;
    }
    var url = `https://media.fromus.org/${file.filename}`;
    var type = '';
    if (ext == '.jpeg' || ext == '.jpg' || ext == '.png') {
        type = 'image';
    } else {
        type = 'pdf';
    }
    var query = 'INSERT INTO posts_media (post_id, path, type) VALUES (?, ?, ?)';
    return new Promise((resolve, reject) => {
        pool.query(query, [post_id, url, type], (err) => {
            if (err) {
                resolve(false);
            } else {
                resolve({ path: url, type: type });
            }
        });
    });
}

function getMedia(post_id) {
    var query = 'SELECT id, path, type FROM posts_media WHERE post_id = ?';
    return new Promise((resolve, reject) => {
        pool.query(query, [post_id], (err, results) => {
            if (err) {
                resolve(false);
            } else {
                resolve(results);
            }
        });
    });
}

function getPost(post_id) {
    var query = `
    SELECT 
    posts.id as post_id, posts.content, posts.date,
    users.name, users.profile_picture, users.uid as username, users.graduation_year,
    likes.count as likes,
    liked.liked as liked,
    comments_count.count as comments_count
    FROM posts posts
    LEFT JOIN users users
    ON posts.user_id = users.id
    LEFT JOIN LATERAL
    (
        SELECT COUNT(*) as liked, post_id
        FROM posts_likes
        WHERE post_id = posts.id
        AND user_id = 0
    )as liked
    ON posts.id = liked.post_id
    LEFT JOIN LATERAL
    (
        SELECT COUNT(*) as count, post_id
        FROM posts_likes
        WHERE post_id = posts.id
        GROUP BY post_id
    )as likes
    ON posts.id = likes.post_id
    LEFT JOIN LATERAL
    (
        SELECT COUNT(*) as count, post_id
        FROM posts_comments
        WHERE post_id = posts.id
        GROUP BY post_id
    ) as comments_count
    ON posts.id = comments_count.post_id
    WHERE posts.id = ?
    LIMIT 1
    `;
    return new Promise((resolve, reject) => {
        pool.query(query, [post_id], (err, results) => {
            if (err) {
                resolve(false);
            } else {
                resolve(results[0]);
            }
        });
    });
}

function getUserPosts(requested_user_id, user_id) {
    var query = `
    SELECT
    users.name as user_name, users.profile_picture, users.uid as user_username, users.id as user_id,
    posts.id as post_id, posts.content, posts.date, posts.issue_id, posts.specific_issue,
    issues.name as issue,
    (SELECT COUNT(*) FROM posts_awareness_points WHERE post_id = posts.id) as awareness_points,
    (SELECT COUNT(*) FROM posts_comments WHERE post_id = posts.id) as comments_count,
    (SELECT COUNT(*) FROM posts_awareness_points WHERE post_id = posts.id AND user_id = ?) as aware
    FROM users users
    LEFT JOIN posts posts
    ON posts.user_id = users.id
    LEFT JOIN issues issues
    ON posts.issue_id = issues.id
    WHERE posts.user_id = ?
    ORDER BY date DESC
    LIMIT 15
    `;
    var param = [];
    return new Promise((resolve, reject) => {
        pool.query(query, [user_id, requested_user_id], (err, results) => {
            if (err) {
                resolve(false);
            } else {
                var modified_results = [];
                results.forEach(post => {
                    post = new postModels.Post(post);
                    modified_results.push(post);
                });
                resolve(modified_results);
            }
        });
    });
}

module.exports = {
    checkPostExistence,
    checkLikeExistence,
    uploadMedia,
    getMedia,
    getUserPosts,
    getPost
};
//import mysql pool
const pool = require('../database');