const express = require('express');
const cookieParser = require('cookie-parser');
const bcrypt = require('bcryptjs');
const moment = require('moment');
const multer = require('multer');
const path = require('path');
const storage = multer.diskStorage({
    destination: (req, file, cb) => {
        cb(null, '../fromus_media');
    },
    filename: (req, file, cb) => {
        cb(null, Date.now() + file.originalname)
    }
})
const upload = multer({
    storage: storage,
    fileFilter: function (req, file, callback) {
        var ext = path.extname(file.originalname);
        if (ext !== '.png' && ext !== '.jpg' && ext !== '.jpeg') {
            return callback(new Error('Only images are allowed'))
        }
        callback(null, true)
    },
});
const userModels = require('../models/user');
const authjs = require('../functions/auth');
const fs = require('fs');

var router = express.Router();
router.use(cookieParser());

router.get('/', async (req, res) => {
    var id = req.cookies.c_id;
    var token = req.cookies.c_t;
    if (!id && !token) {
        res.json({ status: 401, error: 'Failed to Verify', message: 'Invalid Credentials' });
        return;
    }

    var query = 'SELECT * FROM users_tokens WHERE id = ? AND user_id = ?';
    pool.query(query, [token, id], (err, results) => {
        if (err) {
            res.json({ status: 500, error: 'Internal Server Error', message: 'Something Went Wrong' });
        } else if (!results[0]) {
            res.json({ status: 401, error: 'Failed to Verify', message: 'Invalid Credentials' });
        } else {
            res.json({ status: 200, message: 'Ok' });
        }
    });
});

router.get('/t', async (req, res) => {
    res.json(req.cookies);
});

router.post('/login', upload.none(), async (req, res) => {
    var uid = req.body.username;
    var pwd = req.body.password;

    if (!uid || !pwd) {
        res.json({ status: 400, message: 'Please fill in all fields', error: 'Invalid Request' });
        return;
    }

    var uidExistence = await authjs.uidExistence(uid);
    if (uidExistence == false) {
        res.json({ status: 400, message: 'Please enter the correct username and password', error: 'Invalid Credentials' });
        return;
    }

    bcrypt.compare(pwd, uidExistence.pwd, async (err, response) => {
        if (err) {
            res.json({ status: 500, message: "Something went wrong" });
        } else if (response == false) {
            res.json({ status: 400, message: 'Please enter the correct username and password', error: 'Invalid Credentials' });
        } else {
            var genToken = await authjs.generateToken(uidExistence.id);
            if (genToken == false) {
                res.json({ status: 500, message: "Something went wrong", error: "Internal Server Error" });
            } else {
                res.cookie('c_id', uidExistence.id, { domain: ".jamesmadisonalumni.org", expires: new Date(Date.now() + 10000000000), path: '/' });
                res.cookie('c_t', genToken.token, { domain: ".jamesmadisonalumni.org", expires: new Date(Date.now() + 10000000000), path: '/' });
                res.json({ status: 200 })
            }
        }
    });
});

router.post('/logout', upload.none(), async (req, res) => {
    var headers = req.headers;
    if (!headers.origin || headers.origin != 'fromus.org/logout.php') {
        res.json({ status: 400, error: 'Invalid Request' });
        return;
    }
    var id = req.cookies.c_id;
    var token = req.cookies.c_t;
    if (!id || !token) {
        res.json({ status: 400, error: 'Invalid Request' });
        return;
    }

    var query = 'DELETE FROM users_tokens WHERE id = ? AND user_id = ?';
    pool.query(query, [token, id], (err, results) => {
        if (err) {
            res.json({ status: 500 });
        } else {
            res.json({ status: 200 });
        }
    });
});

router.post('/edit', upload.single('pfp'), async (req, res) => {
    var id = req.cookies.c_id;
    var token = req.cookies.c_t;
    var name = req.body.name;
    var email = req.body.email;
    var bio = req.body.bio;
    var pfp = req.file;

    var auth = await authjs.checkToken(id, token);
    if (auth == false) {
        res.json({ status: 401, message: 'Failed to authenticate' });
        return;
    }

    if (!id || !token || !name || !email) {
        res.json({ status: 400, message: "Please fill in all fields" });
        return;
    }

    if (name.length > 150 || name.trim() == 0) {
        res.json({ status: 400, message: "Provide a valid name" });
        return;
    }

    var re = /^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/;
    if (re.test(String(email).toLowerCase()) == false) {
        res.json({ status: 400, message: "Provide a valid email" });
        return;
    }

    if (bio.length > 120) {
        res.json({ status: 400, message: "Provide a valid bio" });
        return;
    }

    var info = {
        name,
        email,
        bio
    };

    if (pfp) {
        info.profile_picture = `https://media.fromus.org/${pfp.filename}`;
    }

    var query = 'UPDATE users SET ? WHERE id = ?';
    pool.query(query, [info, id], (err, results) => {
        if (err) {
            res.json({ status: 500, message: 'Something went wrong', err });
        } else {
            res.json({ status: 200 });
        }
    });

});

module.exports = router;
//import mysql pool
const pool = require('../database');