const express = require('express');
const mysql = require('mysql');
const moment = require('moment');
const bcrypt = require('bcryptjs');
const multer = require('multer');
const cookieParser = require('cookie-parser')
const path = require('path');
const storage = multer.diskStorage({
    destination: (req, file, cb) => {
        cb(null, '../fromus_media');
    },
    filename: (req, file, cb) => {
        cb(null, Date.now() + file.originalname)
    }
})
const upload = multer({ storage: storage });
const userModels = require('../models/user');
const authFunctions = require('../functions/auth');
const postFunctions = require('../functions/posts');
const authjs = require('../functions/auth');
const fs = require('fs');

var router = express.Router();
router.use(cookieParser());

router.get('/info', async (req, res) => {
    var headers = req.headers;
    if (!headers.origin || headers.origin != 'jamesmadisonalumni.org/auth/user.php') {
        res.json({ status: 400, error: 'Invalid Request' });
        return;
    }
    var id = req.cookies.c_id;
    var token = req.cookies.c_t;
    if (!id || !token) {
        res.json({ status: 400, error: 'Invalid Request' });
        return;
    }

    var auth = await authFunctions.checkToken(id, token);
    if (auth == false) {
        res.json({ status: 401, message: 'Failed to authenticate' });
        return;
    }

    var query = `SELECT 
    users.id, users.name, users.profile_picture, users.email, users.bio
    FROM users users
    WHERE users.id = ?
    LIMIT 1`;
    pool.query(query, [id], (err, result) => {
        if (err) {
            res.json({ status: 500, message: 'Internal Server Error', err });
            return;
        }
        res.json({ status: 200, message: 'Success', data: result[0] });
    });
});

router.get('/:id', async (req, res) => {
    var id = req.cookies.c_id;
    var token = req.cookies.c_t;
    if (id && token) {
        var auth = await authFunctions.checkToken(id, token);
        if (auth == true) {
            id == 0;
        }
    }
    var requested_user_id = req.params.id;
    var query = `
    SELECT
    users.id, users.uid as username, users.joined, users.name, users.role_id, users.bio, users.profile_picture,
    roles.name as account_type,
    posts_count.count as posts_count
    FROM users users
    LEFT JOIN roles roles
    ON users.role_id = roles.id
    LEFT JOIN LATERAL
    (
        SELECT COUNT(*) as count, user_id
        FROM posts
        WHERE posts.user_id = users.id
    ) as posts_count
    ON users.id = posts_count.user_id
    WHERE users.id = ?
    LIMIT 1
    `;
    pool.query(query, [requested_user_id], async (err, result) => {
        if (err) {
            res.json({ status: 500, message: 'Something went wrong', err });
        } else {
            var posts = await postFunctions.getUserPosts(requested_user_id, id);

            if (posts === false) {
                res.json({ status: 500, message: 'Sorry, we are unable to load posts' });
            } else {
                for (let i = 0; i < posts.length; i++) {
                    const post_element = posts[i];
                    var media = await postFunctions.getMedia(post_element.post_id);
                    post_element.media = media;
                    post_element.media_count = media.length;
                }
                var user = result[0];
                if (user.posts_count == null) {
                    user.posts_count = 0;
                }
                if (user.awareness_points == null) {
                    user.awareness_points = 0;
                }
                res.json({ status: 200, user: result[0], posts });
            }
        }
    });
});

router.post('/signup', upload.none(), async (req, res) => {
    var body = req.body;
    var name = body.name;
    var email = body.email;
    var uid = body.username;
    var grad_year = body.grad_year;
    var pwd = body.password;
    var pwd_repeat = body.password_repeat;

    if (!name || !email || !uid || !pwd || !pwd_repeat || !grad_year) {
        res.json({ status: 400, error: 'Bad Request', message: 'Please fill in all fields' });
        return;
    }

    if(grad_year > moment().year()){
        res.json({ status:400, error:'Bad Request', message:"Invalid year, choose a year before " + moment().year() });
        return;
    }

    if (name.length > 150) {
        res.json({ status: 400, message: "Enter a shorter name", error: 'Name too long', field: 'name' });
        return;
    }

    var re = /^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/;
    if (re.test(String(email).toLowerCase()) == false || email.length > 150) {
        res.json({ status: 400, message: 'Enter a valid email', error: 'Invalid Email', field: 'email' });
        return;
    }

    re = /\s/g;
    if (re.test(String(uid).toLowerCase()) == true || uid.length > 15) {
        res.json({ status: 400, message: "Enter a valid username", error: 'Invalid Username', field: "username" });
        return;
    }

    if (pwd != pwd_repeat) {
        res.json({ status: 400, message: 'Your passwords do not match' });
        return;
    }

    var uidExistence = await authjs.uidExistence(uid);
    if (uidExistence != false) {
        res.json({ status: 400, message: 'Username is already taken', error: 'Invalid Username', field: 'username' });
        return;
    }

    try {
        const salt = await bcrypt.genSalt();
        const hashedPwd = await bcrypt.hash(pwd, salt);

        const user = {
            uid,
            pwd: hashedPwd,
            joined: moment().format("YYYY-MM-DD HH:mm:ss"),
            name,
            email,
            graduation_year:grad_year,
            profile_picture: 'https://jamesmadisonalumni.org/assets/blank_pfp.webp'
        };

        var query = 'INSERT INTO users SET ?';
        pool.query(query, user, (err, results) => {
            if (err) {
                res.json({ status: 500, message: 'Internal Server Error' });
            } else {
                res.json({ status: 200, message: 'User added', data: { userId: results.insertId } });
            }
        });
    } catch {
        res.json({ status: 500, message: 'Internal Server Error' });
    }
});

module.exports = router;
//import mysql pool
const pool = require('../database');